;---------------------------------------------------------------;
;                                                               ;
; Banchor                                                       ;
; Demon Routines                                                ;
;                                                               ;
;---------------------------------------------------------------;

;------------------------------------------------
; Initialisation
;------------------------------------------------
iniDemon:
        call    clearDemonTable
        ld      hl,D_INI_YX
        ld      (y),hl
        xor     a
        ld      (playerDir),a
        ld      (dCnt),a
        ld      (dFlags),a
        ld      hl,(demon)
        ld      h,0
        dec     l
        add     hl,hl
        ld      de,demonIniTable
        add     hl,de
        call    _ldhlind
        ld      de,finishIniDemon
        push    de
        jp      (hl)
finishIniDemon:
#ifndef INVINCIBLE
        ld      (_@demonDamage),a
#endif
        ld      (_@doDemon),bc
        ld      (_@drawDemon),de
        ld      (_@finishedDemon),hl
        ld      (_@demonBullet),ix

;------------------------------------------------
; Main Loop
;------------------------------------------------
mainDemonLoop:
        ld      hl,frame
        inc     (hl)

        ld      hl,hurt
        ld      a,(hl)
        or      a
        jr      z,$+3
        dec     (hl)

        call    copyBuffers

        ld      a,(attacking)
        or      a
        jr      z,checkPlayerMovementD
        inc     a
        jr      nz,afterPlayerMovementD
checkPlayerMovementD:
        ld      a,KG_ARROW
        call    directIn
        push    af
        bit     DI_UP,a
        call    z,dPlayerUp
        pop     af
        push    af
        bit     DI_DOWN,a
        call    z,dPlayerDown
        pop     af
        push    af
        bit     DI_LEFT,a
        call    z,dPlayerLeft
        pop     af
        push    af
        bit     DI_RIGHT,a
        call    z,dPlayerRight
        pop     af
        inc     a
        jr      z,afterPlayerMovementD
        ld      hl,walkCnt
        inc     (hl)
afterPlayerMovementD:
        ld      a,KG_TOP
        call    directIn
;;; Demon escape key
;        bit     DI_F5,a
;        jp      z,demonFinished
;;;
        bit     DI_2ND,a
        ld      hl,afterAttackingDemon
        jp      z,tryAttacking
        xor     a
        ld      (attacking),a
afterAttackingDemon:

; Move demon
        call    $0000
_@doDemon                               = $-2

; Check sword/demon collisions
        call    loadDemonToCollide2
        ld      a,(attacking)
        or      a
        jr      z,afterCheckSDCollision
        call    loadSwordToCollide1
        call    checkCollision
        jr      nc,afterCheckSDCollision
        ld      de,(attackStrength)
        ld      d,0
        ld      hl,(dHealth)
        or      a
        sbc     hl,de
        ld      (dHealth),hl
        jp      c,demonFinished
afterCheckSDCollision:
; Check player/demon collisions
        call    loadPlayerToCollide1
        call    checkCollision
        jr      nc,afterCheckPDCollision
        ld      a,(hurt)
        or      a
        jr      nz,afterCheckPDCollision
#ifndef INVINCIBLE
        ld      b,$00
_@demonDamage                           = $-1
        call    calculateDamage
        call    decPlayerHearts
        ld      a,INI_HURT
        ld      (hurt),a
#endif
afterCheckPDCollision:
; Check player/demon object collisions
        ld      b,MAX_DEMON_OBJECTS
        ld      ix,demonTable
checkPOCollisions:
        push    bc
        ld      a,(ix+D_DIR)
        or      a
        jr      z,endCheckPOCollisions
        call    loadBulletToCollide2
        call    checkCollision
        jr      nc,endCheckPOCollisions
        ld      a,(hurt)
        or      a
        jr      nz,endCheckPOCollisions
#ifndef INVINCIBLE
        ld      b,ONE_HEART
        call    calculateDamage
        call    decPlayerHearts
        ld      a,INI_HURT
        ld      (hurt),a
#endif
endCheckPOCollisions:
        pop     bc
        ld      de,DEMON_ENTRY_SIZE
        add     ix,de
        djnz    checkPOCollisions

; Draw demon
        call    $0000
_@drawDemon                             = $-2
; Draw demon objects
        call    drawDemonObjects
; Clear left and right sides
        ld      hl,grayMem1
        call    fillSide
        ld      hl,grayMem1+14
        call    fillSide
; Draw player
        call    drawPlayer

        call    showGray

        ld      a,(hearts)
        or      a
        jp      nz,mainDemonLoop
        jp      playerDead

;------------------------------------------------
; Finished
;------------------------------------------------
demonFinished:
        xor     a
        ld      (attacking),a
        ld      (demon),a
        ld      (frame),a
        ld      a,(maxHearts)
        ld      (hearts),a
        ld      hl,(_@drawDemon)
        ld      (_@drawDemonDeath),hl

; Do demon death animation
        ld      b,60
demonDeathAnimLoop:
        push    bc
        ld      a,b
        cp      30
        call    nc,$0000                        ; Only draw demon for first half of animation
_@drawDemonDeath                        = $-2
        pop     bc
        push    bc
        bit     0,b
        jr      nz,noNewDemonExplosion          ; Only do a new explosion animation every 2nd frame
; Start another explosion animation
        ld      b,9
        call    random
        ld      hl,dX
        add     a,(hl)
        push    af
        ld      b,9
        call    random
        ld      hl,dY
        add     a,(hl)
        pop     bc
        ld      c,a
        call    newAnim
noNewDemonExplosion:
        call    drawPlayer
        call    drawAnims
        call    showGray
        call    copyBuffers
        call    updateAnims
        ld      bc,15
        call    wait
        pop     bc
        djnz    demonDeathAnimLoop

; Run routine to get all demon finished values (different for each demon)
        call    $0000
_@finishedDemon                         = $-2
        ld      hl,demonWarp
        ld      (hl),a
        inc     hl
        ld      (hl),b
        inc     hl
        ld      (hl),c
        ex      de,hl
        call    addGold
        ld      a,(ix)                          ; A = Number of change list entries to add
        or      a
        jr      z,noDemonChanges
        ld      b,a
        inc     ix
doDemonChanges:
        push    bc
        ld      a,(ix+0)
        ld      b,(ix+1)
        ld      c,(ix+2)
        call    addToChangeList
        pop     bc
        ld      de,3
        add     ix,de
        djnz    doDemonChanges
noDemonChanges:
        ld      hl,demonWarp-1
        jp      warpToMap

;------------------------------------------------
; drawDemon - Draw 16x16 demon sprite
;
; Input:    HL => Sprite
; Output:   None
;------------------------------------------------
drawDemon:
        ld      b,4
        ld      de,(dy)
        ld      ix,demonDrawTable
drawDemonLoop:
        push    bc
        push    hl
        push    de
        push    ix
        ld      b,d
        ld      c,e
        call    graySprite
        pop     ix
        pop     de
        ld      a,d
        add     a,(ix)
        ld      d,a
        ld      a,e
        add     a,(ix+1)
        ld      e,a
        inc     ix
        inc     ix
        pop     hl
        ld      bc,24
        add     hl,bc
        pop     bc
        djnz    drawDemonLoop
        ret

;------------------------------------------------
; drawDemonObjects - Draw demon objects
;
; Input:    None
; Output:   None
;------------------------------------------------
drawDemonObjects:
        ld      ix,demonTable
        ld      b,MAX_DEMON_OBJECTS
drawDemonObjectsLoop:
        push    bc
        push    ix
        ld      a,(ix+D_DIR)
        or      a
        jr      z,endDrawDemonObjectsLoop
        ld      b,(ix+D_X)
        ld      c,(ix+D_Y)
        ld      hl,$0000
_@demonBullet                           = $-2
        call    graySpriteClip
endDrawDemonObjectsLoop:
        pop     ix
        pop     bc
        ld      de,DEMON_ENTRY_SIZE
        add     ix,de
        djnz    drawDemonObjectsLoop
        ret

;------------------------------------------------
; loadDemonToCollide2 - Load demon info to (collide2)
;
; Input:    None
; Output:   None
;------------------------------------------------
loadDemonToCollide2:
        ld      hl,collide2
        ld      a,(dx)
        ld      (hl),a
        inc     hl
        ld      (hl),16
        inc     hl
        ld      a,(dy)
        ld      (hl),a
        inc     hl
        ld      (hl),16
        ret

;------------------------------------------------
; clearDemonTable - Clear demonTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearDemonTable:
        ld      hl,demonTable
        ld      b,MAX_DEMON_OBJECTS*DEMON_ENTRY_SIZE
        jp      _ld_hl_bz

;------------------------------------------------
; getEmptyDemonEntry - Try to find an empty entry in demonTable
;
; Input:    None
; Output:   HL => Start of entry (if one was found)
;           CA = 1 if no entry was empty
;------------------------------------------------
getEmptyDemonEntry:
        ld      hl,demonTable
        ld      b,MAX_DEMON_OBJECTS
        ld      de,DEMON_ENTRY_SIZE
findEmptyDemonEntry:
        ld      a,(hl)
        or      a
        ret     z
        add     hl,de
        djnz    findEmptyDemonEntry
        scf
        ret

;------------------------------------------------
; moveDemonObjectsNormal - Move all demon objects with standard angle code
;
; Input:   None
; Output:  None
;------------------------------------------------
moveDemonObjectsNormal:
        ld      b,MAX_DEMON_OBJECTS
        ld      ix,demonTable
moveDemonObjectsLoop:
        ld      a,(ix+D_DIR)
        or      a
        jr      z,endMoveDemonObjectsLoop
        call    moveDemonObject
endMoveDemonObjectsLoop:
        ld      de,DEMON_ENTRY_SIZE
        add     ix,de
        djnz    moveDemonObjectsLoop
        ret

;------------------------------------------------
; moveDemonObject - Move a demon object
;
; Input:    IX => Start of object entry
; Output:   IX => Start of object entry
;------------------------------------------------
moveDemonObject:
        ld      l,(ix+D_DIR)
        dec     l
        ld      h,0
        add     hl,hl
        ld      de,bulletOffsets
        add     hl,de
        ld      a,(hl)
        add     a,(ix+D_Y)
        ld      (ix+D_Y),a
        inc     hl
        ld      a,(hl)
        add     a,(ix+D_X)
        ld      (ix+D_X),a
        cp      DSXMIN-5
        jr      c,demonObjectOffScreen
        cp      DSXMAX
        jr      nc,demonObjectOffScreen
        ld      a,(ix+E_Y)
        cp      64
        ret     c
        cp      -5
        ret     nc
demonObjectOffScreen:
        ld      (ix+D_DIR),0
        ret

;------------------------------------------------
; getDemonAngle - Get angle between demon & player
;
; Input:   None
; Output:  A = Angle
;------------------------------------------------
getDemonAngle:
        call    swapDemonXY
        ld      ix,dy-1
        call    getAngle
; Fall through to swapDemonXY and return

;------------------------------------------------
; swapDemonXY - Swap Demon X & Y coordinates
;
; Input:   None
; Output:  None
;------------------------------------------------
swapDemonXY:
        ld      hl,(dy)
        ld      e,l
        ld      l,h
        ld      h,e
        ld      (dy),hl
        ret

;------------------------------------------------
; checkDemonOnScreen - Check to make sure Demon is on-screen properly
;
; Input:   None
; Output:  None
;------------------------------------------------
checkDemonOnScreen:
        ld      hl,dy
        ld      a,(hl)
        cp      6*8
        jr      c,demonOnScreen1
        ld      (hl),6*8
demonOnScreen1:
        cp      8
        jr      nc,demonOnScreen2
        ld      (hl),8
demonOnScreen2:
        inc     hl                              ; HL => Demon X pos
        ld      a,(hl)
        cp      128-16-16-8
        jr      c,demonOnScreen3
        ld      (hl),128-16-16-8
demonOnScreen3:
        cp      16+8
        ret     nc
        ld      (hl),16+8
        ret

;------------------------------------------------
; Initialisation routine pointers
;------------------------------------------------
demonIniTable:
.dw     iniHeath,iniDezemon,iniWendeg,iniBelkath,iniAnazar,iniBanchor

.end
