;---------------------------------------------------------------;
;                                                               ;
; Banchor                                                       ;
; Input/Output Routines                                         ;
;                                                               ;
;---------------------------------------------------------------;

;------------------------------------------------
; waitKey - Wait for a key press
;
; Input:    None
; Output:   A = Key code (_getcsc style)
;------------------------------------------------
waitKey:
        ei
waitKeyLoop:
        halt
        call    _getcsc
        or      a
        jr      z,waitKeyLoop
        ret

;------------------------------------------------
; waitKeyA - Wait for a key press excluding arrow keys
;
; Input:    None
; Output:   A = Key code (_getcsc style)
;------------------------------------------------
waitKeyA:
        call    waitKey
        cp      5
        jr      c,waitKeyA
        ret

;------------------------------------------------
; directIn - Check keypresses on a specified group
;
; Input:    A = Group to check
; Output:   A = Keycodes
;------------------------------------------------
directIn:
        ld      b,a
        ld      a,$FF
        out     (1),a
        ld      a,b
        out     (1),a
        in      a,(1)
        ret

;------------------------------------------------
; contrastUp - Turn contrast up a tad
;
; Author:   David Phillips <david@acz.org>
; Input:    None
; Output:   None
;------------------------------------------------
contrastUp:
        ld      de,$c008
        ld      a,(de)
        inc     a
        cp      32
        ret     z
        out     (2),a
        ld      (de),a
        ret
 
;------------------------------------------------
; contrastDown - Turn contrast down a tad
;
; Author:   David Phillips <david@acz.org>
; Input:    None
; Output:   None
;------------------------------------------------
contrastDown:
        ld      de,$c008
        ld      a,(de)
        or      a
        ret     z
        dec     a
        out     (2),a
        ld      (de),a
        ret

;------------------------------------------------
; setShowHL - Set coords and show HL in _vputs font
;
; Input:    HL = Number to show
;           DE = Coords to load to (_pencol)
;           B = Number of characters to show
; Output:   None
;------------------------------------------------
setShowHL:
        ld      (_pencol),de

;------------------------------------------------
; showHL - Show HL in _vputs font
;
; Input:    HL = Number to show
;           B = Number of characters to show
; Output:   None
;------------------------------------------------
showHL:
        ld      de,string+5
        xor     a
        ld      (de),a
convertHLToAscii:
        dec     de
        call    _divhlby10
        add     a,'0'
        ld      (de),a
        djnz    convertHLToAscii
        ex      de,hl
        jp      _vputs

;------------------------------------------------
; wait - Wait a bit
;
; Input:    BC = How many times to loop
; Output:   None
;------------------------------------------------
wait:
        ei
waitLoop:
        halt
        dec     bc
        ld      a,b
        or      c
        jr      nz,waitLoop
        ret

;------------------------------------------------
; fillSide - Fill a side of the screen
;
; Input:    HL => Start of video memory to fill
; Output:   None
;------------------------------------------------
fillSide:
        ld      b,128
        ld      de,15
fillSideLoop:
        ld      (hl),$FF
        inc     hl
        ld      (hl),$FF
        add     hl,de
        djnz    fillSideLoop
        ret

.end
