;---------------------------------------------------------------;
;                                                               ;
; Banchor                                                       ;
; Main Game Loop                                                ;
;                                                               ;
;---------------------------------------------------------------;

checkAreaName:
        ld      a,(area)                        ; A = New area number
        ld      hl,areaNum                      ; HL => Last area number
        ld      b,(hl)                          ; B = Last area number
        ld      (hl),a                          ; Save new area number
        or      a                               ; Check if it's 0
        jr      z,mainLoop                      ; If so, no area name
        cp      b                               ; Check to see if still in same area
        call    nz,showAreaName                 ; If not, show area name

mainLoop:
        BREAK

        ld      a,(demon)                       ; Check if player is abt to fight a demon
        or      a
        jp      nz,iniDemon                     ; If so, initialise and DO IT!

        ld      hl,frame
        inc     (hl)

        ld      hl,hurt
        ld      a,(hl)
        or      a
        jr      z,$+3
        dec     (hl)

        call    animateMap

        call    copyBuffers

        ld      a,(y)
        add     a,3
        and     %11111000
        add     a,a
        ld      b,a
        ld      a,(x)
        add     a,3
        srl     a
        srl     a
        srl     a
        add     a,b
        ld      (playerOffset),a                ; Save offset in map of player

        ld      a,(attacking)
        or      a
        jr      z,checkPlayerMovement
        inc     a
        jr      nz,afterPlayerMovement          ; Can't move if attacking!
checkPlayerMovement:
        ld      a,KG_ARROW
        call    directIn                        ; Get keypress from keypad
        push    af
        bit     DI_UP,a                         ; Check [UP]
        call    z,playerUp
        pop     af
        push    af
        bit     DI_DOWN,a                       ; Check [DOWN]
        call    z,playerDown
        pop     af
        push    af
        bit     DI_LEFT,a                       ; Check [LEFT]
        call    z,playerLeft
        pop     af
        push    af
        bit     DI_RIGHT,a                      ; Check [RIGHT]
        call    z,playerRight
        pop     af
        inc     a
        jr      z,afterPlayerMovement
        ld      hl,walkCnt
        inc     (hl)
afterPlayerMovement:
        ld      a,KG_1
        call    directIn                        ; Get keypresses from left column
        bit     DI_ALPHA,a                      ; Check [ALPHA]
        jr      nz,afterCheckPeopleChests       ; If not pressed, don't check for people of chests
        call    checkPeople                     ; Check to see if there's a person to talk to
        call    checkChests                     ; Check to see if there's a chest to open
afterCheckPeopleChests:
        ld      a,KG_TOP
        call    directIn                        ; Get keypresses from top row
        bit     DI_MORE,a                       ; Check [MORE]
        jp      z,showInventoryScreen
        bit     DI_EXIT,a                       ; Check [EXIT]
        jp      z,quit
        bit     DI_2ND,a                        ; Check [2nd]
        ld      hl,afterAttacking
        jp      z,tryAttacking
        xor     a
        ld      (attacking),a

afterAttacking:
        ld      a,(attacking)
        or      a
        jr      z,afterCheckStones
        inc     a
        jr      z,afterCheckStones
        ld      a,(ringOfMight)                 ; Check if player has Ring Of Might
        or      a
        jr      z,afterCheckStones              ; If not, can't crush anything
        ld      bc,3                            ; BC = 3 tiles to check
        ld      a,(ringOfThunder)               ; Check if player has Ring Of Thunder
        or      a
        jr      z,$+5                           ; If not, only check stone tiles
        ld      bc,6                            ; Otherwise, 6 tiles to check (3 stone, 3 rock)
        push    bc                              ; Save for later usage
        ld      a,(playerDir)
        add     a,a
        ld      e,a
        ld      d,0
        ld      hl,swordCoords
        add     hl,de
        ld      bc,(y)
        ld      a,b
        add     a,(hl)
        ld      b,a
        ld      a,c
        inc     hl
        add     a,(hl)
        ld      c,a
        ld      (_@stoneCoords),bc
        ld      l,a
        ld      a,b
        call    getTile
        ex      de,hl
        ld      hl,stoneRockTiles
        pop     bc                              ; BC = Number of tiles to check, depending on which Ring player has
        cpir
        jr      nz,afterCheckStones
        ld      hl,replaceStoneRockTiles
        add     hl,bc
        ld      a,(hl)
        ld      (de),a
        ld      b,a
        ld      hl,$0000
_@stoneCoords                           = $-2
        srl     h
        srl     h
        srl     h
        srl     l
        srl     l
        srl     l
        ld      a,l
        add     a,a
        add     a,a
        add     a,a
        add     a,a
        add     a,h
        ld      c,a
        ld      a,b
        call    animateMapSprite
afterCheckStones:
; Check warps
        ld      a,(numWarps)
        or      a
        jr      z,afterCheckWarps
        ld      b,a
        ld      hl,warps
        ld      a,(playerOffset)
        ld      de,4
checkWarps:
        cp      (hl)                            ; Compare playerOffset to warp offset
        jp      z,warpToMap
        add     hl,de
        djnz    checkWarps
afterCheckWarps:
        ld      a,(frame)                       ; Get frame counter
        and     $0F                             ; Only try this on every 16th frame
        call    z,makeEnemy                     ; Try making an enemy

        call    moveBullets                     ; Move enemy bullets
        call    moveEnemies                     ; Move enemies
        call    updateAnims                     ; Update animations

        call    checkPlayerEnemyCollisions      ; Check player/enemy collisions
        call    checkPlayerBulletCollisions     ; Check player/bullet collisions
        call    checkSwordEnemyCollisions       ; Check sword/enemy collisions

        call    drawBullets                     ; Draw enemy bullets
        call    drawEnemies                     ; Draw enemies
        call    drawAnims                       ; Draw animations
        call    drawPlayer                      ; Draw player

        call    showGray                        ; Copy finished frame to LCD

        ld      a,(hearts)
        or      a
        jp      nz,mainLoop                     ; Can only keep playing if still alive!
        jp      playerDead

.end
