;---------------------------------------------------------------;
;                                                               ;
; Banchor                                                       ;
; Talking Routines                                              ;
;                                                               ;
;---------------------------------------------------------------;

;------------------------------------------------
; checkPeople - Check to see if player can talk to a person
;
; Input:    None
; Output:   None
;------------------------------------------------
checkPeople:
        ld      a,(playerDir)
        or      a
        ret     nz                              ; If player not facing up, can't talk to a person
        ld      a,(numPeople)
        or      a
        ret     z                               ; If there's no people on this screen, leave
        ld      b,a                             ; B = Number of ppl on screen

        ld      a,(playerOffset)
        sub     16
        ld      c,a                             ; C = Offset in map
        ld      hl,people                       ; HL => People data
checkPeopleLoop:
        ld      a,c
        cp      (hl)                            ; Check offset
        jr      nz,notTalkingToPerson           ; If not equal, player isn't talking to this person
        inc     hl
        ld      a,(hl)                          ; A = Text message number
        pop     de                              ; Remove now-redundant stack data
        call    textMessage                     ; Talk to the person
        call    grayDrawMap                     ; Re-draw the screen
        jp      mainLoop                        ; And proceed with the game
notTalkingToPerson:
        inc     hl
        inc     hl                              ; HL => Next person
        djnz    checkPeopleLoop
        ret

;------------------------------------------------
; textMessage - Show a text message
;
; Input:    A = Text message to show
; Output:   NZ = 1 if a "YES" choice was made
;------------------------------------------------
textMessage:
        push    af
        call    loadZBDat2                      ; Load "zbdat2" which has talking data/text
        pop     af
        ld      hl,endTextMessage               ; HL => Where to RET to after finished talking
        push    hl                              ; Put address on stack
        ld      hl,talkText
        or      a
        jr      z,textMessageFound
        ld      b,a
findTextMessage:
        ld      a,(hl)
        inc     hl
        inc     a
        jr      nz,findTextMessage
        djnz    findTextMessage
textMessageFound:
        push    hl
        ld      hl,talkMap
        call    grayDrawTileMap
        ld      hl,_plotsscreen
        ld      de,_plotsscreen+1
        ld      bc,1023
        ld      (hl),0
        ldir
        pop     hl

        set     textwrite,(iy+new_grf_flgs)
        ld      de,5*256+6
        ld      b,0
writeTextLine:
        ld      (_pencol),de
        ld      a,(hl)
        inc     a
        jr      z,textMessageFinished
        inc     a
        jr      z,prepareForTextChoice
        push    bc
        call    _vputs
        pop     bc
        inc     b
        ld      a,b
        cp      8
        jr      z,waitNextTextPage
        ld      a,(_penrow)
        add     a,6
        ld      d,a
        ld      e,6
        jr      writeTextLine
prepareForTextChoice:
        ld      hl,53*256+6                     ; HL = New cursor position
        ld      (_pencol),hl                    ; Set it
        ld      hl,strYesNo
        call    _vputs                          ; Write "YES"
        ld      a,114                           ; A = New _pencol position
        ld      (_pencol),a                     ; Set it
        call    _vputs                          ; Write "NO"
        ld      a,$80
textMessageFinished:
        res     textwrite,(iy+new_grf_flgs)
        add     a,$37
        ld      (_@textChoice),a                ; Load either OR A or SCF
; OR the contents of _plotsscreen to buffer1 to write font in dark-grey
        ld      hl,buffer1
        ld      de,_plotsscreen
        ld      bc,1024
copyTextMessage:
        ld      a,(de)
        or      (hl)
        ld      (hl),a
        inc     hl
        inc     de
        dec     bc
        ld      a,c
        or      b
        jr      nz,copyTextMessage
        nop
_@textChoice                            = $-1
        jr      c,noTextChoice
        xor     a
        ld      (choice),a
        call    copyBuffers
        call    showGray
textChoiceLoop:
        ld      a,KG_TOP
        call    directIn                        ; Get keypresses for top row
        bit     DI_F5,a                         ; Was [F5] pressed?
        ret     z                               ; If so, return Z for "NO"
        bit     DI_F1,a                         ; Was [F1] pressed?
        jr      nz,textChoiceLoop               ; If not, loop again
        inc     a                               ; Set NZ
        ret
waitNextTextPage:
        push    hl
        xor     a
        call    textMessageFinished
        pop     hl
        jp      textMessageFound
noTextChoice:
        call    copyBuffers
        call    showGray
        call    waitKey                         ; Wait for a keypress
        cp      GK_ENTER                        ; Was it [ENTER]
        jr      nz,noTextChoice                 ; If not, loop again
        ret
endTextMessage:
        push    af
        call    loadZBDat
        pop     af
        ret

.end
